#--------------------------------------------------------------------------------------------------------------------------------
# - Nom : Script de rotation avec "Anti-Aliasing"
# - Auteur : Krn
# - But : Faire tourner une image en adoussissant les contours.(Moins de crnelage)
# - Utilisation : - rotation(angle) renvoie un bitmap avec un rotation de l'angle indiqu.
#                      - coordonnees_relatives_rotation(angle) renvoir un tableau du type [x,y]
#                            comprenant les nouvelles coordonnes du point haut gauche de l'image
#                            sur l'image toune.
# - Exemple : bitmap.rotation(30) renverra la mme image ayant subit une rotation de 30
#                      et coordonnees_relatives_rotation(30) permettra de dplacer l'image telle que
#                      le point haut-gauche de l'image soit au mme endroit.
#- Precision : Il faut utiliser coordonnees_relatives_rotation(angle) avant rotation(angle).
#                      Sinon, il calcule en prvion d'un rotation future : la taille du bitmap a dj change.
#--------------------------------------------------------------------------------------------------------------------------------


class Bitmap

  attr_accessor :matrice_red
  attr_accessor :matrice_green
  attr_accessor :matrice_blue
  attr_accessor :matrice_alpha

  def rotation(angle)
    #Pr-calcul des cosinus / sinus
    cos = Math::cos(- angle * Math::PI / 180)
    sin = Math::sin(- angle * Math::PI / 180)

    #Calcul des positions des 4 points extrmes
    cos2 = Math::cos( angle * Math::PI / 180)
    sin2 = Math::sin( angle * Math::PI / 180)

    dx1 = 0
    dy1 = 0

    dx2 = self.width * cos2
    dy2 = self.width * sin2

    dx3 = - self.height * sin2
    dy3 = self.height * cos2

    dx4 = self.width * cos2 - self.height * sin2
    dy4 = self.width * sin2 + self.height * cos2

    dgauche = Math.min(Math.min(dx1,dx2),Math.min(dx3,dx4))
    ddroite = Math.max(Math.max(dx1,dx2),Math.max(dx3,dx4))
    dhaut = Math.min(Math.min(dy1,dy2),Math.min(dy3,dy4))
    dbas = Math.max(Math.max(dy1,dy2),Math.max(dy3,dy4))

    #Cration du nouveau bitmap
    nouveau_bitmap = Bitmap.new((dgauche - ddroite).abs + 2, (dhaut - dbas).abs + 2)

    for x in dgauche.to_i - 1...ddroite.to_i + 1
      for y in dhaut.to_i - 1...dbas.to_i + 1
        source_x = x * cos - y * sin
        source_y = x * sin + y * cos

        source_x_ent = source_x.to_i
        source_y_ent = source_y.to_i

        if source_x < 0
          source_x_ent -=1
        end

        if source_y < 0
          source_y_ent -=1
        end

        c1 = self.get_pixel( source_x_ent , source_y_ent ) #haut_gauche => bas_droit
        c2 = self.get_pixel( source_x_ent + 1 , source_y_ent )#haut_droit =>bas_gauche
        c3 = self.get_pixel( source_x_ent , source_y_ent + 1)#bas_gauche => haut_droit
        c4 = self.get_pixel( source_x_ent + 1 , source_y_ent + 1)#bas_droit => haut_gauche

        rapportx1 = source_x - source_x_ent #dx_gauche
        rapporty1 = source_y - source_y_ent#dy_haut
        rapportx2 = source_x_ent + 1 - source_x#dx_droit
        rapporty2 = source_y_ent + 1 - source_y#dy_bas

        #Alpha du pixel
        alpha = c4.alpha * rapportx1 *  rapporty1 + c3.alpha * rapportx2  * rapporty1 + c2.alpha * rapportx1 * rapporty2 + c1.alpha * rapportx2* rapporty2

        #Rapport  alpha

        rapport_alpha_c1 = c1.alpha.to_f / alpha
        rapport_alpha_c2 = c2.alpha.to_f / alpha
        rapport_alpha_c3 = c3.alpha.to_f / alpha
        rapport_alpha_c4 = c4.alpha.to_f / alpha

        #Rouge du pixel
        red     = c4.red * rapport_alpha_c4 * rapportx1 *  rapporty1 + c3.red * rapport_alpha_c3 *  rapportx2  * rapporty1  + c2.red * rapport_alpha_c2 * rapportx1 * rapporty2 + c1.red * rapport_alpha_c1 * rapportx2* rapporty2
        #Vert du pixel
        green = c4.green * rapport_alpha_c4 * rapportx1 *  rapporty1 + c3.green * rapport_alpha_c3 * rapportx2  * rapporty1  + c2.green * rapport_alpha_c2 * rapportx1 * rapporty2 + c1.green * rapport_alpha_c1* rapportx2* rapporty2
        #Bleu du pixel
        blue    = c4.blue * rapport_alpha_c4 * rapportx1 *  rapporty1 + c3.blue * rapport_alpha_c3 * rapportx2  * rapporty1  + c2.blue * rapport_alpha_c2 * rapportx1 * rapporty2 + c1.blue * rapport_alpha_c1 * rapportx2* rapporty2



        nouveau_bitmap.set_pixel(x - dgauche.to_i + 1,y - dhaut.to_i + 1,Color.new(red,green,blue,alpha))
      end
    end
    return nouveau_bitmap
  end

  def coordonnees_relatives_rotation(angle)

    #Calcul des positions des 4 points extrmes
    cos2 = Math::cos( angle * Math::PI / 180)
    sin2 = Math::sin( angle * Math::PI / 180)

    dx1 = 0
    dy1 = 0

    dx2 = self.width * cos2
    dy2 = self.width * sin2

    dx3 = - self.height * sin2
    dy3 = self.height * cos2

    dx4 = self.width * cos2 - self.height * sin2
    dy4 = self.width * sin2 + self.height * cos2

    dgauche = Math.min(Math.min(dx1,dx2),Math.min(dx3,dx4))
    dhaut = Math.min(Math.min(dy1,dy2),Math.min(dy3,dy4))

    #Renvoi sous la forme d'un tableau [x,y]
    return [- dgauche + 1,- dhaut + 1]
  end

end