
#==============================================================================
# Sprite_Character
#------------------------------------------------------------------------------
#==============================================================================
class Sprite_Character < RPG::Sprite
  #--------------------------------------------------------------------------
  # 
  #--------------------------------------------------------------------------
  attr_accessor :character                # 
  #--------------------------------------------------------------------------
  # 
  #     viewport  : 
  #     character :  (Game_Character)
  #--------------------------------------------------------------------------
  def initialize(viewport, character = nil)
    super(viewport)
    @character = character
    update
  end
  #--------------------------------------------------------------------------
  # 
  #--------------------------------------------------------------------------
  def update
    super
    update_main
    update_tile_0
    update_position
    update_apparence
    update_animation
  end
  
  def update_main
    #  
    if @tile_id != @character.tile_id or
        @character_name != @character.character_name or
        @character_hue != @character.character_hue
      #  ID 
      @tile_id = @character.tile_id
      @character_name = @character.character_name
      @character_hue = @character.character_hue
      #  ID 
      if @tile_id >= 384
        update_main_tileset
      else
        update_main_character
      end
    end
  end
  
  def update_main_tileset
    self.bitmap = RPG::Cache.tile($game_map.tileset_name,
      @tile_id, @character.character_hue)
    self.src_rect.set(0, 0, 32, 32)
    self.ox = 16
    self.oy = 32
    #  ID 
  end
  
  def update_main_character
    changer_bitmap(@character.character_name)
  end
  
  def changer_bitmap(bitmap)
    self.bitmap = RPG::Cache.character(bitmap, @character.character_hue)
    @cw = self.bitmap.width / 4
    @ch = self.bitmap.height / 4
    self.ox = @cw / 2
    self.oy = @ch
  end
  
  def update_tile_0
    # 
    if @tile_id == 0
      update_sprite
    end
  end
  
  def update_sprite
    # 
    sx = @character.pattern * @cw
    sy = (@character.direction - 2) / 2 * @ch
    self.src_rect.set(sx, sy, @cw, @ch)
  end
  
  def update_position
    # 
    self.x = @character.screen_x
    self.y = @character.screen_y
    self.z = @character.screen_z(@ch)
  end
  
  def update_apparence
    # Ligne normalement juste avant if @tile_id == 0
    # 
    self.visible = (not @character.transparent)

    # 
    self.opacity = @character.opacity
    self.blend_type = @character.blend_type
    self.bush_depth = @character.bush_depth
  end
  
  def update_animation
    # 
    if @character.animation_id != 0
      animation = $data_animations[@character.animation_id]
      animation(animation, true)
      @character.animation_id = 0
    end
  end
end
